#!/usr/bin/perl -w

# LSB Test Execution Framework
# Tests Execution Module (tests_exec.pl)
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use Fcntl ':flock';

# Check if we need to run manual tests only
if ($_COOKIE{"profile"} and open(PROFILE, $SERVER_PARAM{"APP_DATA"}."/profiles/".$_COOKIE{"profile"})) {
	my $manual_present = 0;
	my $automatic_present = 0;
	while (<PROFILE>) {
		s/[\r\n]//g;
		if (m/^#MANUAL_TESTS:\s*(.*)$/) {
			$manual_present = $1;
		}
		if (m/^#AUTOMATIC_TESTS:\s*(.*)$/) {
			$automatic_present = $1;
		}
	}
	close(PROFILE);
	if ($manual_present and not $automatic_present) {
		# User selected the manual tests only.
		my $profile_path = $SERVER_PARAM{"APP_DATA"}."/profiles/".$_COOKIE{"profile"};
		system($CONFIG{"TESTS_DIR"}."/lsb-tef.pl -f \"$profile_path\" --results-dir=\"".$CONFIG{"RESULTS_DIR"}."\" > /dev/null 2>&1");
		my $tmp = "";
		if (open(FILE_HIST, $CONFIG{"RESULTS_DIR"}."/HISTORY")) {
			flock(FILE_HIST, LOCK_SH);
			while(<FILE_HIST>) {
				$tmp = $_;
			}
			flock(FILE_HIST, LOCK_UN);
			close(FILE_HIST);
			$tmp =~ s/[\r\n]//g;
		}
		print "HTTP/1.0 302 Moved\n";
		print "Location: tests_appbat.pl?test_run=$tmp\n";
		print "Set-Cookie: profile=; path=/;\n\n";
		exit;
	}
}

my $error_text = "";
my $running = 0;
my $cert = "custom";
if (-f "/tmp/tests_output.8888") {
	my $PID = `head -n 1 /tmp/tests_output.8888 2>/dev/null`;
	$PID =~ s/[\r\n]//g;
	$PID =~ m/PID:\s*(\d+),\s*CHLD_PID:\s*(\d+)(\s*\(cert\))?/i;
	$PID = $1;
	my $kill_PID = $2;
	$running = is_process_running($PID);
	$cert = ($3 ? "certification" : "custom");
}
if ($_COOKIE{"profile"}) {
	if ($running) {
		$error_text = "Tests are already running.<br />You cannot run another instance before they are finished.<br />Start watching the current run?<br /><br /><a href=\"#\" onclick=\"javascript:startRefresh(); return false;\">Yes</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"#\" onclick=\"javascript:err_area.style.display='none'; return false;\">No</a>";
		$running = 0;
	}
	else {
		my $profile_path = $SERVER_PARAM{"APP_DATA"}."/profiles/".$_COOKIE{"profile"};
		my $child = fork();
		if (!defined($child)) {
			$error_text = "Cannot create child process!";
		}
		elsif ($child == 0) {
			close(STDOUT);
			exec($CONFIG{"TESTS_DIR"}."/run_tests.pl \"".$CONFIG{"TESTS_DIR"}."\" \"".$CONFIG{"RESULTS_DIR"}."\" \"$profile_path\"");
		}
		else {
			$running = 1;
			if (open(FILE, $profile_path)) {
				while (<FILE>) {
					if (m/^CERTIFY:\s*(\d+)$/) {
						$cert = ($1 ? "certification" : "custom");
						last;
					}
				}
				close(FILE);
			}
		}
	}
}

my $js_run_update = ($running ? "startRefresh();\n" : "");

print "HTTP/1.0 200 OK\n";
print "Content-type: text/html\n";
print "Set-Cookie: profile=; path=/;\n\n";

print_header("Test Execution Console Page", "exec");

print show_error_dlg($error_text);

print <<DATA;

<h1>Console output</h1>
Please, select the user profile to run:&nbsp;&nbsp;&nbsp;
DATA

my $found = 0;
my $profiles_list = "";
if (opendir(DIR, $SERVER_PARAM{"APP_DATA"}."/profiles")) {
	my @files = sort grep !/^[\.~]/, readdir(DIR);
	foreach (@files) {
		if ($_COOKIE{"profile"} and ($_COOKIE{"profile"} eq $_)) {
			$profiles_list .= "    <option value=\"$_\" selected=\"selected\">$_</option>\n";
		}
		else {
			$profiles_list .= "    <option value=\"$_\">$_</option>\n";
		}
		$found = 1;
	}
	closedir(DIR);
}

if ($found) {
	print <<DATA;
  <select name="test_profile" id="test_profile" style="width: 12em;">
$profiles_list  </select>&nbsp;
  <input type="button" name="Start" id="start_button" value="Start" style="width: 6em;" onclick="javascript:startTests();" />&nbsp;
  <input type="button" name="Stop" id="stop_button" value="Stop" style="width: 6em;" onclick="javascript:stopTests();" disabled="disabled" />
DATA
}
else {
	print <<DATA;
  <select name="test_profile_no" id="test_profile_no" style="width: 12em;" disabled="disabled">
    <option value="">&lt;No profiles present&gt;</option>
  </select>&nbsp;
  <input type="button" name="Start" id="start_button_no" value="Start" style="width: 6em;" disabled="disabled" />&nbsp;
  <input type="button" name="Stop" id="stop_button" value="Stop" style="width: 6em;" onclick="javascript:stopTests();" disabled="disabled" />
DATA
}

print <<DATA;
<br /><br />
<table border="0" cellpadding="0" cellspacing="0" style="width: 60em;"><tr><td align="left">
<b><font color="blue"><span id="exec_info">Nothing started</span></font>&nbsp;<span id="exec_status"></span></b> <b><a id="go_to_manual" style="display: none;" href="tests_appbat.pl">Run manual tests</a></b>
</td><td align="right">
<b><a id="view_report" style="display: none;" href="tests_results.pl">View test report</a></b>
</td></tr></table>
<pre id="cmdlog" style="height: 400px; width: 60em; overflow: auto; text-wrap: none; border: 1px solid black; border-color: black; background-color: black; color: lightgreen;">Execute output will go here...</pre>

<script language="javascript" type="text/javascript">
// <![CDATA[
var err_area = document.getElementById('error_msg_area');
var err_text = document.getElementById('error_msg_text');
var cert = '$cert';

// AJAX
var refresh_delay = 1000;

var ajax_srv_url = document.location.protocol + '//' + document.location.host + '/ajax_srv.pl';

var cmdlog = document.getElementById('cmdlog');
var exec_status = document.getElementById('exec_status');
var exec_info = document.getElementById('exec_info');
var go_to_manual = document.getElementById('go_to_manual');
var view_report = document.getElementById('view_report');
if (!cmdlog || !exec_status || !exec_info || !go_to_manual || !view_report)
	alert('Internal error: Cannot find essential form fields! JavaScript will not work correctly.\\nPlease, contact the authors.');

var cmdlog_height;
if (navigator.appName.indexOf("Opera") != -1)
	cmdlog_height = cmdlog.style.pixelHeight;
else
	cmdlog_height = cmdlog.clientHeight;

function startTests() {
	var profile = document.getElementById('test_profile');
	if (profile) {
		if (profile.value) {
			loadXMLDoc(ajax_srv_url + '?action=run_tests&profile=' + profile.value, 'GET', null);
		}
		else {
			alert("Profile is not specified!");
			return;
		}
	}
	else {
		alert("Internal error: Cannot find 'profile' form field!\\nPlease, contact the authors.!");
		return;
	}
}

function startRefresh() {
	var start_button = document.getElementById('start_button');
	if (start_button)
		start_button.disabled = true;
	var stop_button = document.getElementById('stop_button');
	if (stop_button)
		stop_button.disabled = false;
	var test_profile = document.getElementById('test_profile');
	if (test_profile)
		test_profile.disabled = true;
	exec_info.innerHTML = 'Tests are running (' + cert + ')...';
	exec_status.innerHTML = '&nbsp;&nbsp;&nbsp;<img src="images/ajax_progress.gif" width="16" height="16" alt="" />';
	cmdlog.innerHTML = '';
	loadXMLDoc(ajax_srv_url + '?action=get_tests_output&start=0', 'GET', null);
}

function stopTests() {
	loadXMLDoc(ajax_srv_url + '?action=stop_tests&tree=tests', 'GET', null);
}

function stopRefresh() {
	var start_button = document.getElementById('start_button');
	if (start_button)
		start_button.disabled = false;
	var stop_button = document.getElementById('stop_button');
	if (stop_button)
		stop_button.disabled = true;
	var test_profile = document.getElementById('test_profile');
	if (test_profile)
		test_profile.disabled = false;
}

function loadXMLDoc(url, method, post_params) {
	var req;
	err_area.style.display = 'none';
	if (window.XMLHttpRequest) {        // native XMLHttpRequest object
		req = new XMLHttpRequest();
	}
	else if (window.ActiveXObject) {    // IE/Windows ActiveX version
		try {
			req = new ActiveXObject('Msxml2.XMLHTTP');
		}
		catch (e) {
			try {
				req = new ActiveXObject('Microsoft.XMLHTTP');
			}
			catch (e) {
				alert('Sorry, your browser does not support AJAX');
				return false;
			}
		}
	}
	if (!req) {
		alert('Sorry, your browser does not support AJAX');
		return false;
	}

    req.onreadystatechange = processReqChange;
    req.open(method, url, true);
	req.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
	req.setRequestHeader('If-Modified-Since', 'Tue, 01 Jan 1980 00:00:00 GMT');
	req.setRequestHeader('Content-length', (post_params ? post_params.length : 0));
	req.setRequestHeader('Connection', 'close');
    req.send(post_params);

	function processReqChange() {
		if (req.readyState == 4) {     // Finished
	        if (req.status == 200) {
				if (req.responseXML.getElementsByTagName('error').length > 0) {
		            var error_text = req.responseXML.getElementsByTagName('error')[0].childNodes[0].nodeValue;
	            	err_text.innerHTML = error_text;
	            	err_area.style.display = '';
					exec_info.innerHTML = 'Nothing started';
					exec_status.innerHTML = '';
					stopRefresh();
				}
				else {
	            	var fid;
					if (req.responseXML.getElementsByTagName('cert').length > 0) {
						cert = ((req.responseXML.getElementsByTagName('cert')[0].childNodes[0].nodeValue == '0') ? 'custom' : 'certification');
		            }
	            	if (req.responseXML.getElementsByTagName('started').length > 0) {
						setTimeout("startRefresh()", 100);
	            	}
	            	else {
						if (req.responseXML.getElementsByTagName('output').length > 0) {
							var output = '';
							for (var i=0; i<req.responseXML.getElementsByTagName('output').length; ++i) {
								if (req.responseXML.getElementsByTagName('output')[i].childNodes.length > 0)
									output += req.responseXML.getElementsByTagName('output')[i].childNodes[0].nodeValue;
							}
							updateCmdLog(output);
			            }
						if (req.responseXML.getElementsByTagName('fid').length > 0) {
							fid = req.responseXML.getElementsByTagName('fid')[0].childNodes[0].nodeValue;
			            }
						if (req.responseXML.getElementsByTagName('size').length > 0) {
							var sz = req.responseXML.getElementsByTagName('size')[0].childNodes[0].nodeValue;
							setTimeout("loadXMLDoc('" + ajax_srv_url + "?action=get_tests_output&start=" + sz + "&fid=" + fid + "', 'GET', null)", refresh_delay);
						}
						else {
							if (req.responseXML.getElementsByTagName('status').length > 0) {
								exec_info.innerHTML = 'Tests are finished (' + cert + ').';
								exec_status.innerHTML = req.responseXML.getElementsByTagName('status')[0].childNodes[0].nodeValue;
								stopRefresh();
							}
							if (req.responseXML.getElementsByTagName('redirect').length > 0) {
								document.location = 'tests_results.pl?details=' + req.responseXML.getElementsByTagName('redirect')[0].childNodes[0].nodeValue + '&summary=1';
				            }
						}
					}
				}
	        }
	        else {
	            alert('Error in AJAX call:\\n' + req.status + ' (' + req.statusText + ')');
	        }
	    }
	}
}

function updateCmdLog(txt) {
	var scroll_top = cmdlog.scrollTop;
	var scroll_height = cmdlog.scrollHeight;
	var auto_scroll = (scroll_height - cmdlog_height - scroll_top < 10);
	if (typeof(cmdlog.innerText) == "undefined")
		cmdlog.textContent += txt;
	else
		cmdlog.innerText += txt;
	if (auto_scroll) {
		cmdlog.scrollTop = cmdlog.scrollHeight;
		cmdlog.scrollTop = cmdlog.scrollHeight;		// IE fix
	}
}

$js_run_update
// ]]>
</script>
DATA

print_footer();
